
/*****************************************************************************

###ACTION###
Name:  Move
Short: Moves a render object to a new display position.
###END###

*****************************************************************************/

static ERROR RENDER_Move(struct Render *Self, struct acMove *Args)
{
   struct RenderList *list;
   struct acMove move;
   struct acMoveToPoint moveto;
   UBYTE msgbuffer[sizeof(struct Message) + sizeof(struct ActionMessage) + sizeof(struct acMove)];
   struct ActionMessage *action;
   APTR queue;
   LONG index;
   WORD i, mx, my;

   if (!Args) return(ObjectError(Self, ERH_Move, ERR_Args));

   /* Check if other move messages are queued for this object - if so,
   ** do not do anything until the final message is reached.
   */

   if (AccessMemory(GetResource(RES_MESSAGEQUEUE), MEM_READ, &queue) IS ERR_Okay) {
      index = 0;
      while (ScanMessages(queue, &index, MSGID_ACTION, msgbuffer, sizeof(msgbuffer)) IS ERR_Okay) {
         action = (struct ActionMessage *)(msgbuffer + sizeof(struct Message));

         if ((action->ActionID IS AC_MoveToPoint) AND (action->ObjectID IS Self->Head.UniqueID)) {
            ReleaseMemory(queue);
            return(ERR_Okay|ERF_Notified);
         }
         else if ((action->ActionID IS AC_Move) AND (action->SendArgs IS TRUE) AND
                  (action->ObjectID IS Self->Head.UniqueID)) {
            struct acMove *msgmove;
            msgmove = (struct acMove *)(action + 1);
            msgmove->XChange += Args->XChange;
            msgmove->YChange += Args->YChange;
            msgmove->ZChange += Args->ZChange;

            UpdateMessage(queue, ((struct Message *)msgbuffer)->UniqueID, NULL, action, sizeof(struct ActionMessage) + sizeof(struct acMove));

            ReleaseMemory(queue);
            return(ERR_Okay|ERF_Notified);
         }
      }
      ReleaseMemory(queue);
   }

   if (Self->Flags & RNF_STICKY) return(ERR_Failed|ERF_Notified);

   if (Self->Flags & RNF_NOHORIZONTAL) move.XChange = 0;
   else move.XChange = Args->XChange;

   if (Self->Flags & RNF_NOVERTICAL) move.YChange = 0;
   else move.YChange = Args->YChange;

   move.ZChange = 0;

   /*** If there isn't any movement, return immediately ***/

   if ((move.XChange < 1) AND (move.XChange > -1) AND
       (move.YChange < 1) AND (move.YChange > -1)) {
      return(ERR_Failed|ERF_Notified);
   }

   #ifdef DBG_DRAW
      DPrintF("~Move()","[Render:%d:%s] XYZ: %.0f,%.0f,%.0f", Self->Head.UniqueID, Self->Head.Stats->Name, Args->XChange, Args->YChange, Args->ZChange);
   #endif

   /*** Margin/Limit handling ***/

   if (!Self->ParentID) ActionMsg(AC_Move, Self->ScreenID, &move);
   else if ((list = AccessRenderList(ARF_READ, NULL))) {
      i = FindDrawableIndex(list, Self->ParentID);

      /*** Horizontal limit handling ***/

      if (Args->XChange < 0) {
         if ((Self->XCoord + Args->XChange) < Self->LeftLimit) {
            if (Self->XCoord < Self->LeftLimit) move.XChange = 0;
            else move.XChange = -(Self->XCoord - Self->LeftLimit);
         }
      }
      else if (Args->XChange > 0) {
         if ((Self->XCoord + Self->Width) > (list[i].Width - Self->RightLimit)) move.XChange = 0;
         else if ((Self->XCoord + Self->Width + Args->XChange) > (list[i].Width - Self->RightLimit)) {
            move.XChange = (list[i].Width - Self->RightLimit - Self->Width) - Self->XCoord;
         }
      }

      /*** Vertical limit handling ***/

      if (Args->YChange < 0) {
         if ((Self->YCoord + Args->YChange) < Self->TopLimit) {
            if ((Self->YCoord + Self->Height) < Self->TopLimit) move.YChange = 0;
            else move.YChange = -(Self->YCoord - Self->TopLimit);
         }
      }
      else if (Args->YChange > 0) {
         if ((Self->YCoord + Self->Height) > (list[i].Height - Self->BottomLimit)) move.YChange = 0;
         else if ((Self->YCoord + Self->Height + Args->YChange) > (list[i].Height - Self->BottomLimit)) {
            move.YChange = (list[i].Height - Self->BottomLimit - Self->Height) - Self->YCoord;
         }
      }

      /*** Second check: If there isn't any movement, return immediately ***/

      if ((!move.XChange) AND (!move.YChange)) {
         ReleaseRenderList(ARF_READ);
         #ifdef DBG_DRAW
            StepBack();
         #endif
         return(ERR_Failed|ERF_Notified);
      }

      if (Self->prvAnchored IS DRAG_ANCHOR) {
         index = FindDrawableIndex(list, Self->Head.UniqueID);
         mx = list[index].AbsX + move.XChange + Self->prvAnchorX;
         my = list[index].AbsY + move.YChange + Self->prvAnchorY;
      }
      else { mx = 0; my = 0; }

      ReleaseRenderList(ARF_WRITE);

      /* Move the mouse pointer if it is anchored to this drawable.  We must
      ** do this without an active render-list lock in order to avoid the
      ** high risk of dead-locking when software cursors are used.
      */

      if (Self->prvAnchored IS DRAG_ANCHOR) {
         SetCursorPos(mx, my);
      }

      /*** Move the graphics layer ***/

      MoveLayer(Self, Self->XCoord + move.XChange, Self->YCoord + move.YChange);
   }
   else {
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(ObjectError(Self, ERH_Move, ERR_LockFailed)|ERF_Notified);
   }

/* These lines cause problems for the resizing of offset render objects.
   if (Self->prvDimensions & (DMF_FIXED_XOFFSET|DMF_RELATIVE_XOFFSET)) Self->prvXOffset += move.XChange;
   if (Self->prvDimensions & (DMF_FIXED_YOFFSET|DMF_RELATIVE_YOFFSET)) Self->prvYOffset += move.YChange;
*/

   /*** Notify subscribers ***/

   NotifySubscribers(Self, AC_Move, &move, NULL);

   moveto.XCoord = Self->XCoord;
   moveto.YCoord = Self->YCoord;
   moveto.ZCoord = 0;
   NotifySubscribers(Self, AC_MoveToPoint, &moveto, NULL);

   #ifdef DBG_DRAW
      StepBack();
   #endif

   return(ERR_Okay|ERF_Notified);
}

/*****************************************************************************
** Function: MoveLayer()
*/

static void MoveLayer(struct Render *Self, LONG XCoord, LONG YCoord)
{
   struct RenderList *list;
   struct acMove move;
   struct ClipRectangle abs, old;
   LONG destx, desty, xchange, ychange;
   BYTE volatilegfx;
   WORD vindex, index, parentindex, i;

   /*** If the coordinates are unchanged, do nothing ***/

   if ((XCoord IS Self->XCoord) AND (YCoord IS Self->YCoord)) return;

   if (!(Self->Head.ObjectFlags & NF_INITIALISED)) {
      Self->XCoord = XCoord;
      Self->YCoord = YCoord;
      return;
   }

   xchange = XCoord - Self->XCoord;
   ychange = YCoord - Self->YCoord;

   /*** This subroutine is used if the render object is screen-based ***/

   if (!Self->ParentID) {
      move.XChange = XCoord - Self->XCoord;
      move.YChange = YCoord - Self->YCoord;
      move.ZChange = 0;
      if (ActionMsg(AC_Move, Self->ScreenID, &move) IS ERR_Okay) {
         Self->XCoord = XCoord;
         Self->YCoord = YCoord;
         UpdateRenderList(Self);
      }
      else ObjectError(Self, ERH_Move, ERR_Failed);
      return;
   }

   /* If the window is invisible, set the new coordinates and return
   ** immediately.
   */

   if (!(Self->Flags & RNF_VISIBLE)) {
      Self->XCoord = XCoord;
      Self->YCoord = YCoord;
      UpdateRenderList(Self);
      return;
   }

   if ((list = AccessRenderList(ARF_WRITE, NULL))) {
      if ((index = FindDrawableIndex(list, Self->Head.UniqueID)) IS -1) {
         ReleaseRenderList(ARF_WRITE);
         return;
      }

      old.left   = list[index].Left;
      old.top    = list[index].Top;
      old.right  = list[index].Right;
      old.bottom = list[index].Bottom;

      destx = old.left + XCoord - Self->XCoord;
      desty = old.top  + YCoord - Self->YCoord;

      if (Self->Flags & RNF_REGION) {
         /*** Drawing code for region based render objects.  This is achieved by redrawing the parent ***/

         #ifdef DBG_DRAW
            DPrintF("~MoveLayer:","[%d] Using region redraw technique.", Self->Head.UniqueID);
         #endif

         Self->XCoord = XCoord;
         Self->YCoord = YCoord;
         UpdateRenderList(Self);

         abs = old;

         /*** Merge the old and new rectangular areas into one big rectangle ***/

         if (list[index].Left   < abs.left)   abs.left   = list[index].Left;
         if (list[index].Top    < abs.top)    abs.top    = list[index].Top;
         if (list[index].Right  > abs.right)  abs.right  = list[index].Right;
         if (list[index].Bottom > abs.bottom) abs.bottom = list[index].Bottom;

         if ((abs.right - abs.left) * (abs.bottom - abs.top) > list[index].Width * list[index].Height * 3) {
            /* Split the redraw into two parts (redraw section exposed from the
            ** move, then redraw the region at its new location)
            */
            RedrawDrawable(Self->ParentID, old.left, old.top, old.left+list[index].Width, old.top+list[index].Height, NULL);
            RedrawDrawable(Self->ParentID, list[index].Left, list[index].Top, list[index].Right, list[index].Bottom, NULL);

            ExposeDrawable(Self->ParentID, old.left, old.top, old.left+list[index].Width, old.top+list[index].Height, EXF_ABSOLUTE|EXF_REDRAW_VOLATILE_OVERLAP);
            ExposeDrawable(Self->ParentID, list[index].Left, list[index].Top, list[index].Right, list[index].Bottom, EXF_ABSOLUTE|EXF_REDRAW_VOLATILE_OVERLAP);
         }
         else {
            /* If the region has only moved a little bit, redraw it in one shot */
            RedrawDrawable(Self->ParentID, abs.left, abs.top, abs.right, abs.bottom, NULL);
            ExposeDrawable(Self->ParentID, abs.left, abs.top, abs.right, abs.bottom, EXF_ABSOLUTE|EXF_REDRAW_VOLATILE_OVERLAP);
         }

         #ifdef DBG_DRAW
            StepBack();
         #endif
      }
      else {
         /* Since we do not own our graphics buffer, we need to
         ** shift the content in the buffer first, then send an
         ** expose message to have the changes displayed on screen.
         **
         ** This process is made more complex if there are siblings
         ** above and intersecting our drawable.
         */

         if ((parentindex = FindDrawableIndex(list, Self->ParentID)) != -1) {
            volatilegfx = CheckVolatile(list, index);

            /*** Check if this object or its children are volatile ***/

            if ((volatilegfx) OR (SlowVideoBlit(Self->ScreenID) IS TRUE)) {
               /* This routine can be used as a slow but very reliable
               ** way of moving a solid or volatile render object.  If the render object
               ** is bitmap buffered and lies in video RAM then it becomes
               ** preferable to use this method, but video RAM buffers are rare.
               */

               #ifdef DBG_DRAW
                  DPrintF("~MoveLayer:","[%d] Using simple expose technique.  Volatile: %d", Self->Head.UniqueID, volatilegfx);
               #endif

               Self->XCoord = XCoord;
               Self->YCoord = YCoord;
               UpdateRenderList(Self);

               if (((volatilegfx) AND (!(Self->Flags & RNF_POSTCOMPOSITE))) OR
                   (list[index].BitmapID IS list[parentindex].BitmapID)) {
                  RedrawDrawable(Self->Head.UniqueID, destx, desty, destx+Self->Width, desty+Self->Height, NULL);
               }

               ExposeDrawable(Self->Head.UniqueID, 0, 0, Self->Width, Self->Height, EXF_CHILDREN|EXF_REDRAW_VOLATILE_OVERLAP);

               /*** Expose underlying graphics ***/

               for (vindex=index+1; list[vindex].Level > list[index].Level; vindex++);
               glVolatileIndex = vindex;
               RedrawNonIntersect(Self->ParentID, (struct ClipRectangle *)(&list[index].Left), &old,
                  (list[index].BitmapID IS list[parentindex].BitmapID) ? IRF_SINGLE_BITMAP : -1,
                  EXF_CHILDREN|EXF_REDRAW_VOLATILE);
               glVolatileIndex = 0;

               #ifdef DBG_DRAW
                  StepBack();
               #endif
            }
            else {
               /* If there is a software cursor in the way, remove it before we shift the graphics.  This
               ** is done by turning the cursor invisible and exposing the region under the cursor in
               ** order to take it off the display.
               */

               #ifdef DBG_DRAW
                  DPrintF("~MoveLayer:","[%d] Using video blitting technique.", Self->Head.UniqueID, volatilegfx);
               #endif

               glIgnoreCursor = TRUE; /* Force MoveContent() to ignore the cursor */

               for (i=0; (list[i].ObjectID) AND (!(list[i].Flags & RNF_CURSOR)); i++);
               if ((list[i].ObjectID) AND (list[i].Flags & RNF_VISIBLE)) {
                  if ((list[index].Left < list[i].Right) AND (list[index].Top < list[i].Bottom) AND
                      (list[index].Bottom > list[i].Top) AND (list[index].Right > list[i].Left)) {
                     list[i].Flags &= ~RNF_VISIBLE;
                     ExposeDrawable(Self->Head.UniqueID, list[i].Left, list[i].Top, list[i].Right, list[i].Bottom, EXF_ABSOLUTE);
                     list[i].Flags |= RNF_VISIBLE;
                  }
               }

               Self->XCoord = XCoord;
               Self->YCoord = YCoord;
               UpdateRenderList(Self);

               MoveGraphics(Self, list, index,
                  old.left, old.top,       /* Source */
                  old.left + Self->Width,
                  old.top + Self->Height,
                  list[index].Left,        /* Destination */
                  list[index].Top,
                  list[parentindex].Left,  /* Clip coordinates */
                  list[parentindex].Top,
                  list[parentindex].Right,
                  list[parentindex].Bottom,
                  MCF_INCLUDE_CHILDREN | MCF_REDRAW_UNDERLYING);

               glIgnoreCursor = FALSE;

               #ifdef DBG_DRAW
                  StepBack();
               #endif
            }
         }
      }

      ReleaseRenderList(ARF_WRITE);
   }
}
