#ifndef  MODULES_SCREEN_H
#define  MODULES_SCREEN_H

/*
**   screen.h
**
**   (C) Copyright 2003-2004 Rocklyte Systems.
**       All Rights Reserved.
*/

#ifndef  MAIN_H
#include <pandora/main.h>
#endif

#ifndef  GRAPHICS_POINTER_H
#include <pandora/graphics/pointer.h>
#endif

#define MODVERSION_SCREEN ((FLOAT)1.0)

struct ScreenInfoV1 {
   OBJECTID ScreenID;
   LONG Flags;
   WORD Width;
   WORD Height;
   WORD BitsPerPixel;
   WORD BytesPerPixel;
   LONG AmtColours;
   LARGE AccelFlags;
};

typedef struct ScreenInfoV2 {
   OBJECTID ScreenID;
   LONG Flags;
   WORD Width;
   WORD Height;
   WORD BitsPerPixel;
   WORD BytesPerPixel;
   LONG AmtColours;
   LARGE AccelFlags;
   struct {
      UBYTE  RedShift;
      UBYTE  BlueShift;
      UBYTE  GreenShift;
      UBYTE  AlphaShift;
      UBYTE  RedMask;
      UBYTE  GreenMask;
      UBYTE  BlueMask;
      UBYTE  AlphaMask;
      UBYTE  RedPos;
      UBYTE  GreenPos;
      UBYTE  BluePos;
      UBYTE  AlphaPos;
   } PixelFormat;
};

typedef struct ScreenInfoV3 {
   OBJECTID ScreenID;  /* Object ID related to the screen display */
   LONG Flags;         /* Screen flags (RNF_VISIBLE etc) */
   WORD Width;         /* Width */
   WORD Height;        /* Height */
   union {
      WORD BitsPerPixel;
      WORD Depth;
   };
   WORD BytesPerPixel;
   LONG AmtColours;
   LARGE AccelFlags;
   struct {               /* Same structure as struct ColourFormat */
      UBYTE  RedShift;    /* Right shift value */
      UBYTE  BlueShift;
      UBYTE  GreenShift;
      UBYTE  AlphaShift;
      UBYTE  RedMask;     /* The unshifted mask value (ranges from 0x00 to 0xff) */
      UBYTE  GreenMask;
      UBYTE  BlueMask;
      UBYTE  AlphaMask;
      UBYTE  RedPos;      /* Left shift/positional value */
      UBYTE  GreenPos;
      UBYTE  BluePos;
      UBYTE  AlphaPos;
   } PixelFormat;
   FLOAT MinRefresh;     /* Minimum refresh rate */
   FLOAT MaxRefresh;     /* Maximum refresh rate */
   FLOAT RefreshRate;    /* Recommended refresh rate */
   LONG Index;           /* Display mode ID (internal) */
};

typedef struct ScreenInfoV3 SCREENINFO;

struct CursorInfo {
   LONG Width;         /* Maximum cursor width for custom cursors */
   LONG Height;        /* Maximum cursor height for custom cursors */
   LONG Flags;         /* Informative flag settings */
   WORD BitsPerPixel;  /* Preferred bits-per-pixel setting for custom cursors */
};

#ifdef __snap__

typedef struct SNAPOptimal {
   void (*PutPixel)(N_int32, N_int32);
   void (*SetPixelColour)(GA_color);
   void (*DrawLine)(N_int32, N_int32, N_int32, N_int32, N_int32);
   N_int32 (*DrawLineClip)(N_int32 x1,N_int32 y1,N_int32 x2,N_int32 y2,N_int32 drawLast,N_int32 clipLeft,N_int32 clipTop,N_int32 clipRight,N_int32 clipBottom);
   void (*SetLineColour)(GA_color);
   void (*SetLineClipColour)(GA_color);
   void (*SetMonoColour)(GA_color);
   void (*DrawRect)(N_int32, N_int32, N_int32, N_int32);
   void (*DrawRectLin)(N_int32 dstOfs,N_int32 dstPitch,N_int32 left,N_int32 top,N_int32 width,N_int32 height,GA_color color,N_int32 mix);
   void (*SetRectColour)(GA_color);
   void (*SrcTransBlt)(N_int32 srcLeft,N_int32 srcTop,N_int32 width,N_int32 height,N_int32 dstLeft,N_int32 dstTop,N_int32 mix,GA_color transparent);
   void (*BitBlt)(N_int32 srcLeft,N_int32 srcTop,N_int32 width,N_int32 height,N_int32 dstLeft,N_int32 dstTop,N_int32 mix);
   void (*BitBltLin)(N_int32 srcOfs,N_int32 srcPitch,N_int32 srcLeft,N_int32 srcTop,N_int32 width,N_int32 height,N_int32 dstLeft,N_int32 dstTop,N_int32 mix);
   void (*SrcTransBltSys)(void *srcAddr,N_int32 srcPitch,N_int32 srcLeft,N_int32 srcTop,N_int32 width,N_int32 height,N_int32 dstLeft,N_int32 dstTop,N_int32 mix,GA_color transparent,N_int32 flipY);
   void (*SrcTransBltLin)(N_int32 srcOfs,N_int32 srcPitch,N_int32 srcLeft,N_int32 srcTop,N_int32 width,N_int32 height,N_int32 dstLeft,N_int32 dstTop,N_int32 mix,GA_color transparent);
   void (*BitBltSys)(void *srcAddr,N_int32 srcPitch,N_int32 srcLeft,N_int32 srcTop,N_int32 width,N_int32 height,N_int32 dstLeft,N_int32 dstTop,N_int32 mix,N_int32 flipY);
   void (*ClipMonoImageSys)(N_int32 x,N_int32 y,N_int32 width,N_int32 height,N_int32 byteWidth,N_uint8 *image,N_int32 transparent,N_int32 clipLeft,N_int32 clipTop,N_int32 clipRight,N_int32 clipBottom);
} GA_optimalFuncs;

struct SNAPGlobals {
   APTR Device;                   /* Device address */
   GA_modeInfo      VideoMode;    /* Current video mode information */
   GA_initFuncs     Init;
   GA_bufferFuncs   Buffer;
   GA_cursorFuncs   Cursor;
   GA_driverFuncs   Driver;
   GA_2DStateFuncs  gsState;
   GA_2DRenderFuncs gsDraw;
   GA_2DStateFuncs  gaState;
   GA_2DRenderFuncs gaDraw;
   GA_optimalFuncs  Optimal;
   GA_DPMSFuncs     gsDPMS;
   UBYTE HWCursor;
   UBYTE CursorVisible;
   WORD  CursorX;
   WORD  CursorY;
   OBJECTID LastBitmapID;
   MEMORYID CursorMID;
};

typedef enum {
   SNAP_INIT = 1,
   SNAP_DRIVER,
   SNAP_BUFFER,
   SWSNAP_2DSTATE,
   SWSNAP_2DRENDER,
   HWSNAP_2DSTATE,
   HWSNAP_2DRENDER,
   HWSNAP_CURSOR,
   SNAP_EXPORTS,
   SNAP_OPTIMAL
} SNAPFunctions;

#endif

#define CRF_HARDWARE 0x00000001

/*****************************************************************************/

#define ACF_VIDEOBLIT    0x0000000000000001
#define ACF_SOFTWAREBLIT 0x0000000000000002

/*****************************************************************************
** Function Table.
*/

#ifndef LOCAL_SCREEN

struct ScreenBase {
   ERROR (*GetScreenInfo)(OBJECTID, SCREENINFO *, LONG);
   LONG  (*ScreenDisplayType)(void);
   ERROR (*SetCursor)(OBJECTID, LONG, LONG, STRING, OBJECTID, LONG *);
   ERROR (*RestoreCursor)(LONG, OBJECTID);
   ERROR (*GetCursorPos)(LONG *, LONG *);
   ERROR (*SetCursorPos)(LONG, LONG);
   ERROR (*GetRelativeCursorPos)(OBJECTID, LONG *, LONG *);
   ERROR (*GetCursorInfo)(struct CursorInfo *, LONG);
   ERROR (*SetCustomCursor)(OBJECTID, LONG, struct Bitmap *, LONG, LONG, OBJECTID, LONG *);
   struct Pointer * (*AccessPointer)(void);
   ERROR (*ScanDisplayModes)(STRING Filter, SCREENINFO **, LONG);

#ifdef __snap__
   /* These functions provided only for the Bitmap module */
   ERROR (*GetVideoMode)(GA_modeInfo **);
   ERROR (*LockGraphics)(void);
   void  (*UnlockGraphics)(void);
   GA_devCtx * (*GetSNAPDevice)(void);
   ERROR (*AllocVideo)(LONG, LONG, GA_buf **);
   void  (*FreeVideo)(GA_buf *);
   ERROR (*GetSNAPFunctions)(LONG, APTR *);
#endif
};

#define AccessPointer()                (ScreenBase->AccessPointer())
#define AllocVideo(a,b,c)              (ScreenBase->AllocVideo(a,b,c))
#define FreeVideo(a)                   (ScreenBase->FreeVideo(a))
#define GetCursorInfo(a,b)             (ScreenBase->GetCursorInfo(a,b))
#define GetCursorPos(a,b)              (ScreenBase->GetCursorPos(a,b))
#define GetRelativeCursorPos(a,b,c)    (ScreenBase->GetRelativeCursorPos(a,b,c))
#define GetScreenInfo(a,b,c)           (ScreenBase->GetScreenInfo(a,b,c))
#define GetSNAPDevice()                (ScreenBase->GetSNAPDevice())
#define GetSNAPFunctions(a,b)          (ScreenBase->GetSNAPFunctions(a,b))
#define GetVideoMode(a)                (ScreenBase->GetVideoMode(a))
#define LockGraphics()                 (ScreenBase->LockGraphics())
#define UnlockGraphics()               (ScreenBase->UnlockGraphics())
#define RestoreCursor(a,b)             (ScreenBase->RestoreCursor(a,b))
#define ScanDisplayModes(a,b,c)        (ScreenBase->ScanDisplayModes(a,b,c))
#define ScreenDisplayType()            (ScreenBase->ScreenDisplayType())
#define SetCursor(a,b,c,d,e,f)         (ScreenBase->SetCursor(a,b,c,d,e,f))
#define SetCursorPos(a,b)              (ScreenBase->SetCursorPos(a,b))
#define SetCustomCursor(a,b,c,d,e,f,g) (ScreenBase->SetCustomCursor(a,b,c,d,e,f,g))

#else

static struct Pointer * AccessPointer(void);
static ERROR GetCursorPos(LONG *, LONG *);
static ERROR GetScreenInfo(OBJECTID, SCREENINFO *, LONG);
static ERROR GetRelativeCursorPos(OBJECTID, LONG *, LONG *);
static ERROR GetCursorInfo(struct CursorInfo *, LONG);
static ERROR RestoreCursor(LONG, OBJECTID);
static ERROR ScanDisplayModes(STRING, SCREENINFO *, LONG);
static LONG ScreenDisplayType(void);
static ERROR SetCursor(OBJECTID, LONG, LONG, STRING, OBJECTID, LONG *);
static ERROR SetCursorPos(LONG, LONG);
static ERROR SetCustomCursor(OBJECTID, LONG, struct Bitmap *, LONG, LONG, OBJECTID, LONG *);

#endif /* LOCAL_SCREEN */

#define ReleasePointer(a)    (ReleaseObject(a))

#endif /* MODULES_SCREEN_H */
