#ifndef SYSTEM_ACTIONS_H
#define SYSTEM_ACTIONS_H TRUE

/*
**  actions.h
**
**  (C) Copyright 2000-2004 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** Structures for defining method and action invocations.
*/

struct ActionArray {
   LONG ActionCode;
   APTR Routine;
};

struct MethodArray {
   LONG   MethodID;
   APTR   Routine;               /* ERROR (*Routine)(OBJECTPTR, APTR); */
   STRING Name;
   struct FunctionField *Args;
   LONG   StructureSize;
};

/****************************************************************************
** Macro based actions.
*/

#define Free(a)         (Action(AC_Free, a, 0))
#define SetRead(a,b,c)  a.Buffer=(b);   a.Length=(c);
#define SetSeek(a,b,c)  a.Position=(b); a.Offset=(c);
#define SetWrite(a,b,c) a.Buffer=(b);   a.Length=(c);

/*****************************************************************************
** Seek positions
*/

#define POS_START   0
#define POS_CURRENT 1
#define POS_END     2

#define SEEK_START   POS_START
#define SEEK_CURRENT POS_CURRENT
#define SEEK_LAST    POS_END

/*****************************************************************************
** MoveToPoint and ScrollToPoint flags.
*/

#define STF_X 0x00000001
#define STF_Y 0x00000002
#define STF_Z 0x00000004

#define MTF_X 0x00000001
#define MTF_Y 0x00000002
#define MTF_Z 0x00000004

/*****************************************************************************
** Clipboard modes.
*/

#define CLIP_CUT   0x00000001
#define CLIP_COPY  0x00000002
#define CLIP_PASTE 0x00000004

/*****************************************************************************
** Action and Notification Structures
*/

struct acActionNotify     { ACTIONID ActionID; OBJECTID ObjectID; APTR Args; LONG Size; ERROR Error; LONG Time; };
struct acClipboard        { LONG Mode; };
struct acCopyData         { OBJECTID DestinationID; };
struct acCustom           { LONG Number; STRING String; };
struct acDataChannel      { OBJECTID ObjectID; LONG DataType; LONG Version; APTR Buffer; LONG Size; LONG TotalEntries; };
struct acDraw             { DOUBLE XCoord; DOUBLE YCoord; DOUBLE Width; DOUBLE Height; };
struct acEventMessage     { LONG EventID; OBJECTID ObjectID; LARGE Integer; APTR Buffer; LONG Size; };
struct acFieldNotify      { FIELD FieldID; OBJECTID ObjectID; };
struct acGetField         { STRING Field; STRING Buffer; LONG Size; };
struct acGetUnlistedField { STRING Field; STRING Buffer; LONG Size; };
struct acMove             { DOUBLE XChange; DOUBLE YChange; DOUBLE ZChange; };
struct acMoveToPoint      { DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; LONG Flags; };
struct acNewChild         { OBJECTID NewChildID; };
struct acNewOwner         { OBJECTID NewOwnerID; LONG ClassID; };
struct acRead             { BYTE *Buffer; LONG Length; LONG Result; };
struct acRedimension      { DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; DOUBLE Width; DOUBLE Height; DOUBLE Depth; };
struct acRename           { STRING Name; };
struct acResize           { DOUBLE Width; DOUBLE Height; DOUBLE Depth; };
struct acSaveImage        { OBJECTID DestinationID; CLASSID ClassID; };
struct acSaveToObject     { OBJECTID DestinationID; CLASSID ClassID; };
struct acScroll           { DOUBLE XChange; DOUBLE YChange; DOUBLE ZChange; };
struct acScrollToPoint    { DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; LONG Flags; };
struct acSeek             { DOUBLE Offset; LONG Position; };
struct acSetField         { STRING Field; STRING Value; };
struct acSetUnlistedField { STRING Field; STRING Value; };
struct acTimer            { LARGE TimeElapsed; LARGE TotalElapsed; };
struct acUserClick        { LONG ButtonFlags; DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; };
struct acUserClickRelease { OBJECTID OverObject; DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; DOUBLE XChange; DOUBLE YChange; DOUBLE ZChange; union { LONG ButtonFlags; LONG Buttons; }; };
struct acUserClickRepeat  { OBJECTID OverObject; DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; union { LONG ButtonFlags; LONG Buttons; }; };
struct acUserMovement     { OBJECTID OverObject; DOUBLE XCoord; DOUBLE YCoord; DOUBLE ZCoord; DOUBLE XChange; DOUBLE YChange; DOUBLE ZChange; };
struct acWrite            { BYTE *Buffer; LONG Length; LONG Result; };

/*****************************************************************************
** Action Macros
*/

#define acActivate(a)                (Action(AC_Activate,(a),NULL))
#define acClear(a)                   (Action(AC_Clear,(a),NULL))
#define acClipboard(a,b)             (ActionTags(AC_Clipboard,(a),(b)))
#define acClosingTag(a)              (Action(AC_ClosingTag,(a),NULL))
#define acDeactivate(a)              (Action(AC_Deactivate,(a),NULL))
#define acDisable(a)                 (Action(AC_Disable,(a),NULL))
#define acDraw(a,b,c,d,e)            (ActionTags(AC_Draw,(a),(DOUBLE)(b),(DOUBLE)(c),(DOUBLE)(d),(DOUBLE)(e)))
#define acDataChannel(a,b,c,d,e,f,g) (ActionTags(AC_DataChannel,(a),(b),(c),(d),(e),(f),(g)))
#define acEnable(a)                  (Action(AC_Enable,(a),NULL))
#define acFlush(a)                   (Action(AC_Flush,(a),NULL))
#define acFocus(a)                   (Action(AC_Focus,(a),NULL))
#define acFree(a)                    (Action(AC_Free,(a),NULL))
#define acHide(a)                    (Action(AC_Hide,(a),NULL))
#define acInit(a)                    (Action(AC_Init,(a),NULL))
#define acLostFocus(a)               (Action(AC_LostFocus,(a),NULL))
#define acMove(a,b,c,d)              (ActionTags(AC_Move,(a),(DOUBLE)(b),(DOUBLE)(c),(DOUBLE)(d)))
#define acMoveToBack(a)              (Action(AC_MoveToBack,(a),NULL))
#define acMoveToFront(a)             (Action(AC_MoveToFront,(a),NULL))
#define acOpeningTag(a)              (Action(AC_OpeningTag,(a),NULL))
#define acRefresh(a)                 (Action(AC_Refresh, (a), NULL))
#define acRename(a,b)                (ActionTags(AC_Rename,(a),(b)))
#define acReset(a)                   (Action(AC_Reset,(a),NULL))
#define acResize(a,b,c,d)            (ActionTags(AC_Resize,(a),(DOUBLE)(b),(DOUBLE)(c),(DOUBLE)(d)))
#define acSaveToObject(a,b,c)        (ActionTags(AC_SaveToObject,(a),(b),(c)))
#define acScroll(a,b,c,d)            (ActionTags(AC_Scroll,(a),(DOUBLE)(b),(DOUBLE)(c),(DOUBLE)(d)))
#define acShow(a)                    (Action(AC_Show,(a),NULL))
#define acSort(a)                    (Action(AC_Sort,(a),NULL))

#define acActivateID(a)              (ActionMsg(AC_Activate,(a),NULL))
#define acClosingTagID(a)            (ActionMsg(AC_ClosingTag,(a),NULL))
#define acDisableID(a)               (ActionMsg(AC_Disable,(a),NULL))
#define acEnableID(a)                (ActionMsg(AC_Enable,(a),NULL))
#define acFlushID(a)                 (ActionMsg(AC_Flush,(a),NULL))
#define acFocusID(a)                 (ActionMsg(AC_Focus,(a),NULL))
#define acFreeID(a)                  (ActionMsg(AC_Free,(a),NULL))
#define acHideID(a)                  (ActionMsg(AC_Hide,(a),NULL))
#define acInitID(a)                  (ActionMsg(AC_Init,(a),NULL))
#define acMoveToBackID(a)            (ActionMsg(AC_MoveToBack,(a),NULL))
#define acMoveToFrontID(a)           (ActionMsg(AC_MoveToFront,(a),NULL))
#define acOpeningTagID(a)            (ActionMsg(AC_OpeningTag,(a),NULL))
#define acShowID(a)                  (ActionMsg(AC_Show,(a),NULL))

#define acDataXML(a,b,c)             (ActionTags(AC_DataChannel,(a),NULL,DATA_XML,1,(b),(c),1))
#define acDataText(a,b,c)            (ActionTags(AC_DataChannel,(a),NULL,DATA_TEXT,1,(b),(c),1))

#define FreeObject(a)                (Action(AC_Free,(a),NULL))

#ifndef LOCAL_KERNEL

#ifndef MODULES_KERNEL_H
#include <pandora/modules/kernel.h>
#endif

extern struct KernelBase *KernelBase;

static inline ERROR acMoveToPoint(APTR Object, DOUBLE X, DOUBLE Y, DOUBLE Z, LONG Flags)
{
   struct acMoveToPoint moveto;
   moveto.XCoord = X;
   moveto.YCoord = Y;
   moveto.ZCoord = Z;
   moveto.Flags  = Flags;
   return(Action(AC_MoveToPoint, Object, &moveto));
}

static inline ERROR acMoveToPointID(OBJECTID ObjectID, DOUBLE X, DOUBLE Y, DOUBLE Z, LONG Flags)
{
   struct acMoveToPoint moveto;
   moveto.XCoord = X;
   moveto.YCoord = Y;
   moveto.ZCoord = Z;
   moveto.Flags  = Flags;
   return(ActionMsg(AC_MoveToPoint, ObjectID, &moveto));
}

static inline ERROR acRead(APTR Object, APTR Buffer, LONG Bytes, LONG *Read)
{
   struct acRead read;
   ERROR error;
   read.Buffer = Buffer;
   read.Length = Bytes;
   if ((error = Action(AC_Read, Object, &read)) IS ERR_Okay) {
      if (Read) *Read = read.Result;
      return(ERR_Okay);
   }
   else {
      if (Read) *Read = 0;
      return(error);
   }
}

static inline ERROR acResizeID(OBJECTID ObjectID, DOUBLE Width, DOUBLE Height, DOUBLE Depth)
{
   struct acResize resize;
   resize.Width  = Width;
   resize.Height = Height;
   resize.Depth  = Depth;
   return(ActionMsg(AC_Resize, ObjectID, &resize));
}

static inline ERROR acWrite(APTR Object, APTR Buffer, LONG Bytes)
{
   struct acWrite write;
   write.Buffer = Buffer;
   write.Length = Bytes;
   return(Action(AC_Write, Object, &write));
}

static inline ERROR acDataChannelID(OBJECTID ObjectID, OBJECTID SenderID, LONG Datatype, LONG Version, APTR Data, LONG Size, LONG Entries)
{
   struct acDataChannel channel;
   channel.ObjectID     = SenderID;
   channel.DataType     = Datatype;
   channel.Version      = Version;
   channel.Buffer       = Data;
   channel.Size         = Size;
   channel.TotalEntries = Entries;
   return(ActionMsg(AC_DataChannel, ObjectID, &channel));
}

#endif

#endif /* SYSTEM_ACTIONS_H */
