#ifndef SYSTEM_MODULE_H
#define SYSTEM_MODULE_H TRUE

/*
**  module.h
**
**  (C) Copyright 1996-2001 Rocklyte Systems.
**      All Rights Reserved.
**
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

#ifndef MODULES_KERNEL_H
#include <pandora/modules/kernel.h>
#endif

/*****************************************************************************
** Function list array structure.
*/

struct Function {
   APTR Address;
   STRING Name;
   struct FunctionField *Args;
};

/*****************************************************************************
** Table-Type definitions.
*/

#define MHF_JMP_LVO       0x00000001  /* LVO jump type (Amiga standard) */
#define MHF_JMP_STRUCTURE 0x00000002  /* Base structure (Linux, C/C++ standard) DEFAULT */
#define MHF_STATIC        0x00000004  /* Keep the module code in memory */

#define JMP_DEFAULT MHF_JMP_STRUCTURE
#define JMP_BASE    MHF_JMP_STRUCTURE
#define MHF_JMP_DEFAULT JMP_DEFAULT
#define MHF_JMP_BASE    JMP_BASE

/*****************************************************************************
** Module file header.
*/

#define MODULE_HEADER_V1 0x4D4F4401

#define MODULE_HEADER EXPORT struct ModHeader ModHeader

struct ModHeader {
   LONG   Version;
   ERROR  (*Init)(OBJECTPTR, struct KernelBase *);
   void   (*Close)(OBJECTPTR);
   ERROR  (*Open)(OBJECTPTR);
   ERROR  (*Expunge)(void);
   LONG   Flags;                  /* Special flags, type of function table wanted from the object kernel */
   struct Function *DefaultList;  /* Pointer to default function list */
   LONG   CPUNumber;              /* CPU that this module is compiled for (lower byte) */
   FLOAT  ModVersion;             /* Version of this module */
   FLOAT  MinKernelVersion;       /* Minimum Kernel version required */
   STRING Author;                 /* Author of the module */
   STRING Copyright;              /* Copyright details */
   STRING Date;                   /* Date of compilation */
   STRING Name;                   /* Name of the module */
};

/*****************************************************************************
** Registered CPU ID's.
*/

enum {
   CPU_68000=1,
   CPU_68010,
   CPU_68020,
   CPU_68030,
   CPU_68040,
   CPU_68060,
   CPU_I286,
   CPU_I386,
   CPU_I486,
   CPU_I586,
   CPU_I686
};

#endif /* SYSTEM_MODULE_H */
